@@ -1,8 +1,9 @@
 """ First step of the algorithm."""
 
 import sys
-from .preprocessingMerge import mergeQuotations, mergeNamedEntityTag
+from .preprocessingMerge import Word, mergeQuotations, mergeNamedEntityTag
 from nltk.stem.wordnet import WordNetLemmatizer
+from nltk.corpus import wordnet
 
 class DependenciesTree:
     """
@@ -10,7 +11,7 @@ class DependenciesTree:
         It is a group of words of the initial sentence.
     """
     def __init__(self, word, namedentitytag='undef', dependency='undef', child=None, parent=None):
-        self.wordList = [(word[:word.rindex('-')],int(word[word.rindex('-')+1:]))] # words of the node
+        self.wordList = [Word(word[:word.rindex('-')],int(word[word.rindex('-')+1:]))] # words of the node
         self.namedEntityTag = namedentitytag 
         self.dependency = dependency # dependency from self to its parent
         self.child = child or [] # children of self
@@ -25,10 +26,10 @@ class DependenciesTree:
         t=''
         if(self.namedEntityTag != 'O' and self.namedEntityTag != 'undef'):
             t+= " [{0}]".format(self.namedEntityTag)
-        s+="\t\"{0}\"[label=\"{1}{2}\",shape=box];\n".format(self.wordList[0][0]+str(self.wordList[0][1]),w,t)
+        s+="\t\"{0}\"[label=\"{1}{2}\",shape=box];\n".format(self.wordList[0].word+str(self.wordList[0].index),w,t)
         # Adding definitions of the edges
         for n in self.child:
-            s+="\t\"{0}\" -> \"{1}\"[label=\"{2}\"];\n".format(self.wordList[0][0]+str(self.wordList[0][1]),n.wordList[0][0]+str(n.wordList[0][1]),n.dependency)
+            s+="\t\"{0}\" -> \"{1}\"[label=\"{2}\"];\n".format(self.wordList[0].word+str(self.wordList[0].index),n.wordList[0].word+str(n.wordList[0].index),n.dependency)
         # Recursive calls
         for n in self.child:
             s+=n.string()
@@ -51,17 +52,17 @@ class DependenciesTree:
             c.parent = self
         if mergeWords:
           self.wordList = other.wordList + self.wordList
-          self.wordList.sort(key = lambda x: x[1])
+          self.wordList.sort(key = lambda x: x.index)
         if other.parent:
             other.parent.child.remove(other)
-        other.wordList = [("merged",0)]
+        other.wordList = None
 
     def getWords(self):
         """
             concatenate all strings of the node (in wordList)
         """
-        self.wordList.sort(key = lambda x: x[1]) 
-        return ' '.join(x[0] for x in self.wordList)
+        self.wordList.sort(key = lambda x: x.index) 
+        return ' '.join(x.word for x in self.wordList)
 
 def computeEdges(r,nameToNodes):
     """
@@ -91,14 +92,16 @@ def computeTags(r,nameToNodes):
         Take in input a piece of the result produced by StanfordNLP, and the
         map from names to nodes.
     """
-    index=1
+    index=0
     # Computation of the tags of the nodes
     for word in r['words']:
+        index+=1
         if word[0].isalnum() or word[0] == '$' or  word[0] == '%':
             w=word[0]+'-'+str(index) # key in the nameToNodes map
-            index+=1
             try:
                 n = nameToNodes[w]
+                assert len(n.wordList) == 1
+                n.wordList[0].pos = word[1]['PartOfSpeech']
                 if word[1]['NamedEntityTag'] != 'O':
                     n.namedEntityTag = word[1]['NamedEntityTag']
             except KeyError:        # this node does not exists (e.g. 'of' preposition)
@@ -112,20 +115,70 @@ def initText(t,s):
     for c in t.child:
         initText(c,s)
 
+def nounify(verbWord):
+    """ 
+        Transform a verb to the closest noun: die -> death
+        From George-Bogdan Ivanov on StackOverflow: http://stackoverflow.com/a/16752477/4110059
+    """
+    verb_synsets = wordnet.synsets(verbWord, pos="v")
+    # Word not found
+    if not verb_synsets:
+        return []
+    # Get all verb lemmas of the word
+    verb_lemmas=[]
+    for s in verb_synsets:
+        verb_lemmas += [l for l in s.lemmas() if s.name().split('.')[1] == 'v']
+    # Get related forms
+    derivationally_related_forms = [(l, l.derivationally_related_forms()) \
+                                    for l in    verb_lemmas]
+    # Filter only the nouns
+    related_noun_lemmas = []
+    for drf in derivationally_related_forms:
+        related_noun_lemmas += [l for l in drf[1] if l.synset().name().split('.')[1] == 'n']
+    # Extract the words from the lemmas
+    words = [l.name() for l in related_noun_lemmas]
+    len_words = len(words)
+    # Build the result in the form of a list containing tuples (word, probability)
+    result = [(w, float(words.count(w))/len_words) for w in set(words)]
+    result.sort(key=lambda w: -w[1])
+    # return the word with highest probability
+    return result[0][0]
+
+def nounifyExcept(verbWord):
+    """
+        Transform a verb to the closest noun: die -> death
+        Hard-coded exceptions (e.g. be, have, do, bear...)
+    """
+    nounifyException = {
+        'be' : 'identity',
+        'have' : 'possession',
+        'do' : 'process',
+        'bear' : 'birth',
+        'live' : 'residence',
+        'direct' : 'director'
+    }
+    try:
+        return nounifyException[verbWord]
+    except KeyError:
+        return nounify(verbWord)
+
 def normalizeWord(word,lmtzr):
     """
         Apply lemmatization to the given word.
     """
-    result=lmtzr.lemmatize(word,'n')
-    if len(result)<len(word):
-        return result
-    return lmtzr.lemmatize(word,'v')
+    if(word.pos and word.pos[0] == 'N'):
+        word.word=lmtzr.lemmatize(word.word,'n')
+        return
+    if(word.pos and word.pos[0] == 'V'):
+        word.word=nounifyExcept(lmtzr.lemmatize(word.word,'v'))
+        return
 
 def normalize(t,lmtzr):
     for c in t.child:
         normalize(c,lmtzr)
     if t.namedEntityTag == 'undef':
-        t.wordList = list((normalizeWord(w[0],lmtzr),w[1]) for w in t.wordList)
+        for w in t.wordList:
+            normalizeWord(w,lmtzr)
 
 def computeTree(r):
     """
@@ -142,5 +195,5 @@ def computeTree(r):
     mergeQuotations(tree,r) # quotation merging
     mergeNamedEntityTag(tree) # NER merging
     lmtzr = WordNetLemmatizer()
-    normalize(tree,lmtzr)
+    normalize(tree,lmtzr) # normalize words (lemmatization + nounify nouns)
     return tree
